@@ -16,27 +16,20 @@
 #include "util.h"
 
 extern "C" {
-	void sub_matrix( double A[], double sub_A[], int sub[], int *p_sub, int *p  );
-
-	void sub_matrix_upper( double A[], double sub_A[], int sub[], int *p_sub, int *p  );
-
 	void sub_row_mins( double A[], double sub_A[], int *sub, int *p );
 
-	void sub_rows_mins( double A[], double sub_A[], int *row, int *col, int *p );
-
-	void sub_cols_mins( double A[], double sub_A[], int *row, int *col, int *p );
-
 	void sub_matrices1( double A[], double A12[], double A22[], int *sub, int *p );
 	
 	void sub_matrix_22( double A[], double A22[], int *sub, int *p );
 
-	void sub_matrices( double A[], double A11[], double A21[], double A22[], int *row, int *col, int *p );
-
-	void sub_matrices_inv( double A[], double A11_inv[], double A21[], double A22[], int *row, int *col, int *p );
-	
 	void inverse( double A[], double A_inv[], int *p );
 
 	void cholesky( double A[], double U[], int *p );
+	
+	void update_sigma( double sigma[], int *sub, double K_11_inv[], double K_11_inv_X_K_12[], double *gam, int *p );
+	
+	void rmvnorm_chol( double sample[], double mean[], double chol_sig[], int *p );
+	    
 }
 
 #endif
